//CCVisu is a tool for co-change visualization.
//This file is part of CCVisu. 

//Copyright (C) 2005  Dirk Beyer

//CCVisu is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public License
//as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//CCVisu is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public License
//along with CCVisu; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//Please find the GNU Lesser General Public License in file
//../license_lgpl.txt or http://www.gnu.org/licenses/lgpl.txt

//Dirk Beyer    (firstname.lastname@epfl.ch)
//Ecole Polytechnique Fdrale de Lausanne (EPFL), Switzerland


package ccvisu;

import java.awt.Button;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/*****************************************************************
 * Frame implementation for displaying the layout on the screen device.
 * Used by WriterDataGraphicsDISP.
 * @version  $Revision: 1.8 $; $Date: 2005/08/22 12:30:56 $
 * @author   Dirk Beyer
 *****************************************************************/
public class ScreenDisplay extends Frame {

  /*****************************************************************
   * Canvas implementation for displaying the layout on the screen.
   * @author   Dirk Beyer
   *****************************************************************/
  private class MyCanvas extends Canvas {
    private Frame parent;
	private WriterDataGraphicsDISP writer;
	
    private Label vertexName;

    /**
     * Constructor.
     * @param parent The parent frame.
     * @param writer The writer that uses this object to draw on.
     *               The paining is deligated to the writer object.
     */
    private MyCanvas(ScreenDisplay parent,
                     WriterDataGraphicsDISP writer) {
      this.parent = parent;
	  this.writer = writer;
    }

	/*****************************************************************
	 * Draws the layout layout on the screen.
	 * @param area  The graphics area for drawing.
     *****************************************************************/
    public void paint(Graphics area) {
	  // Size info.
	  setSize(parent.getSize());
	  int xSize = getSize().width  - getInsets().left - getInsets().right;
	  int ySize = getSize().height - getInsets().top  - getInsets().bottom;

      writer.writeDISP(Math.min(xSize,ySize), 
                       area, 
                       getSize().width, 
		               getSize().height, 
                       getInsets().left);
    } // method paint

  }; // class MyCanvas
  
  /** Canvas for graphics.*/
  private MyCanvas canvas;

  /**
   * Constructor.
   * @param writer      The writer that uses this frame to display the layout.
   * @param backColor   Background color.
   * @param anim        If true, the layout is already displayed
   *                    while the minimizer is still improving it,
   *                    and a simple mouse click on the canvas 
   *                    updates the current layout on the screen.
   *                    If false, the layout is displayed only after
   *                    it is minimized.
   */
  public ScreenDisplay(WriterDataGraphicsDISP writer,
                       Color backColor,
                       boolean anim) {
    // The frame.
    setTitle("Visualization");      // Set window title.
    setLocation(100, 100);          // Set initial window position.
    setSize(1024, 768);             // Set windows size.

    // The graphics canvas.
    canvas = new MyCanvas(this, writer);
	canvas.setBackground(backColor);
    // Add canvas to the frames content pane.
    add(canvas);
    setVisible(true);

    // The dialog (extra window), for the vertex names, etc.
    Dialog vertexNameDialog = new Dialog(this);
    vertexNameDialog.setTitle("Vertex name");
    vertexNameDialog.setLocation(100,0);
    vertexNameDialog.setResizable(true);
    vertexNameDialog.setSize(1200, 50);
    vertexNameDialog.setLayout(new GridLayout(3,1));

	Button saveButton = new Button("Save layout");
    vertexNameDialog.add(saveButton);

    String labelText = "Click on vertex to annote with name, click again to remove.";
	if (anim) {
	  labelText = "Click to update the layout. " + labelText;
	}
	vertexNameDialog.add(new Label(labelText));

	canvas.vertexName = new Label(
	  "Vertex names appear here when mouse is moved."
	  + "                                                           "
	  + "                                                           ");
    vertexNameDialog.add(canvas.vertexName);
 
    vertexNameDialog.pack();
    vertexNameDialog.show();

    // Adds MouseListener for action event ''Save button pressed''.
    // Opens a dialog for choosing the file to which the layout is saved.
    saveButton.addActionListener(
	  new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		  FileDialog fileDialog = new FileDialog(canvas.parent, 
                                                 "Save layout", 
                                                 FileDialog.SAVE);
		  fileDialog.setResizable(true);
		  fileDialog.pack();
		  fileDialog.show();

		  if (fileDialog.getDirectory() != null && fileDialog.getFile() != null) {
		    String fileName = fileDialog.getDirectory() + fileDialog.getFile();
			canvas.writer.writeLAY(fileName);
		  }
		}
	  }
	);

	// Adds MouseMotionListener for mouse event ''Mouse moved on vertex''.
    // Show the name(s) of the vertex(vertices) in the vertexNameDialog.
	canvas.addMouseMotionListener(
	  new MouseMotionAdapter() {
		public void mouseMoved(MouseEvent evt) {
		  int xPos = (int) evt.getPoint().getX();
		  int yPos = (int) evt.getPoint().getY();
		  canvas.vertexName.setText(canvas.writer.getVertexNames(xPos, yPos));
		}
	  }
	);

    // Adds MouseListener for mouse event ''Mouse clicked on vertex''.
	// Draw the name(s) of the vertex(vertices) as annotation on the canvas.
    canvas.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent evt) {
		  int xPos = (int) evt.getPoint().getX();
		  int yPos = (int) evt.getPoint().getY();
		  canvas.writer.toggleVertexNames(xPos, yPos);
		  canvas.repaint();
        }
      }
    );

	// Dialog: Adds WindowListener for window event ''Close''.
	vertexNameDialog.addWindowListener(
	  new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		  dispose(); // Close.
		  System.exit(0);
		}
	  }
	);

	// Frame: Adds WindowListener for window event ''Close''.
	addWindowListener(
	  new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		  dispose(); // Close.
		  System.exit(0);
		}
	  }
	);

  } // constructor
  

  /**
   * Repaint the drawing on the frame, i.e., deligate to canvas.
   */
  public void repaint() {
    this.canvas.repaint();
  } // method repaint


}; // class frame
